/* @pjs pauseOnBlur=true; */
//=============================================================================
//  carte de champs (chp E, equipotentielles, lignes de champ, carte d'intensité) pour une distribution à une charge
//  ©J.Roussel -- 29 Octobre 2015

int OX, OY;//Origine du repère
ArrayList lignes,equipot;
boolean afficheMagnitude=false;
boolean afficheControl=false;


void setup() {
  size(360,360);
  smooth();
  ellipseMode(CENTER);
  OX=width/2;//Repère centré au milieu
  OY=height/2;//Repère centré au milieu
  lignes = new ArrayList();
}

void draw() { 
  background(64); 
  drawChampE();
   for (int i = lignes.size()-1; i >= 0; i--) { 
    LigneDeChamp LDC = (LigneDeChamp) lignes.get(i);
    LDC.display();
  }
  if(afficheMagnitude){map_champE(0.5);}
  if(afficheControl){affiche_control(0);}
  draw_charge();
}

 void draw_charge(){
  fill(255);
  ellipse(OX,OY,20,20);
  stroke(64);
  line(OX-5,OY,OX+5,OY);
  line(OX,OY-5,OX,OY+5);
 }

//Champ E(x,y) (en pixel par frame)
// ATTENTION ! FAIRE: 
// x=xx-OX
// y=yy-OY
PVector champE(float xx, float yy) {
  float rr;
  if(abs(xx-OX)<1 && abs(yy-OY)<1){return new PVector(0,0);}  
  else{rr=pow(sq(xx-OX)+sq(yy-OY),1.5);return new PVector(100*(xx-OX)/rr,100*(yy-OY)/rr);}
}



//tracé du vecteur champ électrique
void drawChampE() {  
  float magnitude;
  strokeWeight(1);
  stroke(128);  
  float arrowsize = 4;
  float len=15;
  for (int i=0;i<width;i+=30)
    for (int j=0;j<height;j+=30){
      if(champE(i,j).mag()!=0){
        pushMatrix();// Translate to location to render vector
        translate(i, j);// Vector heading to get direction (pointing up is a heading of 0)
        rotate(champE(i, j).heading2D()); // Scale it to be bigger or smaller if necessary
        //magnitude=champE(i,j).mag()*10000;
        // Draw three lines to make an arrow
        line(0, 0, len, 0);
        line(len, 0, len-arrowsize, +arrowsize/2);
        line(len, 0, len-arrowsize, -arrowsize/2);
        popMatrix();
      }
    }
}

 //tracé de la carte d'intensité en fausses couleurs. 
  void map_champE(float valeur_max) {  
    int R, G, B;
    float z;
    color c1;
	strokeWeight(2);
    for (int i=0;i<width;i+=2) {
      for (int j=0;j<height;j+=2) {
        //z=min(((champE(i,j).mag())/(valeur_max)),1);
		z=map(sqrt(champE(i,j).mag()),0,valeur_max,0,1);
        c1=echelleCouleur(z);
        stroke(c1);
        point(i, j);   
      }
    }
  }
 
  
 color echelleCouleur(float z){
   float b;
   float R,G,B;
   if(z<=0.25) {b=0;}else if(z>=0.57) {b=1;} else {b=z/0.32-0.78125;}
   R=255*b;
   if(z<=0.42) {b=0;}else if(z>=0.92) {b=1;}else {b=2*z-0.84;}
   G=255*b;
   if(z<=0.42) {b=4*z;}else if(z>=0.92) {b=z/0.08-11.5;}else {b=-2*z+1.84;}
   B=255*b;
  return color(R, G, B,170);
 }
 
 color echelleCouleur2(float z){
   float R,G,B;
   R=int(255*sqrt(z));
   G=int(255*z*z*z);
   B=int(255*sin(2*PI*z));
   return color(R,G,B);
 }
 
void affiche_control() {
   textSize(12);
   textAlign(LEFT,TOP);
   fill(255);
   textLeading(14);
   text("Clic to draw a field line \nPress space bar to display intensity map \nPress R to reset\nPress I to hide panel",14,14);
 }
 
void mousePressed() {
    lignes.add(new LigneDeChamp(mouseX, mouseY, 2));
}

void resetA(){
	for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};
}

void changeA(){
	afficheMagnitude=!afficheMagnitude;
}

void keyPressed() {
  if (key == ' ') {afficheMagnitude=!afficheMagnitude;}
  if (key == 'R' || key == 'r') {for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};} 
  if (key == 'I' || key == 'i'){afficheControl=!afficheControl;} 
}

class LigneDeChamp {
  float x0;
  float y0;
  float pas;
  
LigneDeChamp(float posX, float posY, int ppas) {
    x0=posX;
    y0=posY;
    pas= ppas;
  }
//méthode d'euler
void display() {
  strokeWeight(2);
  stroke(0,153,204,150);
  float xx1,yy1,xx2,yy2;
  float angle;
  xx1=x0;
  yy1=y0;
  xx2=x0;
  yy2=y0;
  pushMatrix();
  translate(xx1,yy1);
  rotate(champE(xx1,yy1).heading2D());
  beginShape(TRIANGLES);
  vertex(0,3);
  vertex(8,0);
  vertex(0,-3);
  endShape();
  //line(10, 0, 4, 3);
  //line(10, 0, 4, -3);
  rotate(-champE(xx1,yy1).heading2D());
  while(!StopTrace(xx1,yy1)){
    angle=champE(xx1,yy1).heading2D();
    xx1+=pas*cos(angle);
    yy1+=pas*sin(angle);
    line(0,0,pas*cos(angle),pas*sin(angle));
    translate(pas*cos(angle),pas*sin(angle));
  }
  popMatrix();
  pushMatrix();
  translate(xx2,yy2);
  while(!StopTrace(xx2,yy2)){
   angle=champE(xx2,yy2).heading2D();
   xx2+=pas*cos(angle+PI);
   yy2+=pas*sin(angle+PI);
   line(0,0,pas*cos(angle+PI),pas*sin(angle+PI));
   translate(pas*cos(angle+PI),pas*sin(angle+PI));
  }
  popMatrix();
}

Boolean StopTrace(float x,float y){
  if((x<width && x>0) && (y<height && y>0) && dist(x,y,OX,OY)>10){return false;}
  else{return true;}
}
}